@@ -20,9 +20,11 @@ from typing import (
     Callable,
     Dict,
     Tuple,
+    Union,
 )
 from urllib.parse import urljoin
 
+import jwt
 from gql import Client
 from qctrlcommons.auth import (
     BearerTokenAuth,
@@ -55,6 +57,20 @@ from ..scripts_utils import (
 LOGGER = logging.getLogger(__name__)
 
 
+def _valid_refresh_token(refresh_token: str) -> bool:
+    """
+    Verify refresh_token signature to ensure it can be used without requiring
+    new credentials.
+    """
+    try:
+        jwt.decode(
+            refresh_token, options={"verify_signature": False, "verify_exp": True}
+        )
+        return True
+    except jwt.InvalidTokenError:
+        return False
+
+
 def get_jwt_token(api_root: str) -> Tuple[str, str, Callable]:
     """Get Jwt token from file storage.
 
@@ -73,15 +89,19 @@ def get_jwt_token(api_root: str) -> Tuple[str, str, Callable]:
     file_path = _credentials_file_path(api_root)
     access_token, refresh_token = _parse_credentials_file(file_path)
 
-    def token_observer(jwt):
+    if not _valid_refresh_token(refresh_token):
+        file_path = _credentials_file_path(api_root, force_authentication=True)
+        access_token, refresh_token = _parse_credentials_file(file_path)
+
+    def token_observer(tokens):
         return write_auth_file(
-            jwt["access_token"], jwt["refresh_token"], file_path=file_path
+            tokens["access_token"], tokens["refresh_token"], file_path=file_path
         )
 
     return access_token, refresh_token, token_observer
 
 
-def _parse_credentials_file(file_path: str) -> tuple:
+def _parse_credentials_file(file_path: Union[str, Path]) -> tuple:
     """Parses a credentials file and returns a tuple of (access_token,
     refresh_token) to be used for login.
 
@@ -113,20 +133,21 @@ def _parse_credentials_file(file_path: str) -> tuple:
     return data["access_token"], data["refresh_token"]
 
 
-def _credentials_file_path(api_root: str) -> Path:
+def _credentials_file_path(api_root: str, force_authentication: bool = False) -> Path:
 
     env_file_path = os.environ.get("QCTRL_AUTHENTICATION_CREDENTIALS", "")
 
     if env_file_path:
-        return Path(env_file_path)
+        file_path = Path(env_file_path)
+    else:
+        file_path = file_path_for_url(api_root)
 
-    default_file_path = file_path_for_url(api_root)
-    if not default_file_path.exists():
+    if not file_path.exists() or force_authentication:
         gql_annonymous_client = create_gql_client(api_root)
         create_token_client = CreateTokenQuery(gql_annonymous_client)
         interactive_authentication(client=create_token_client, api_root=api_root)
 
-    return default_file_path
+    return file_path
 
 
 def create_client_auth(
