@@ -49,11 +49,14 @@ class ResultMixin:
     @property
     def progress(self):
         """The progress of the action."""
-        return self.action.progress
+        return self.action.progress if self.action else 0
 
     @property
     def is_completed(self) -> bool:
         """Checks if action is completed."""
+        if not self.status:
+            return False
+
         return self.status not in (self.PENDING, self.STARTED, self.RETRY)
 
     @property
