@@ -13,7 +13,10 @@
 # pylint:disable=missing-module-docstring
 import logging
 from collections import OrderedDict
-from enum import Enum
+from enum import (
+    Enum,
+    EnumMeta,
+)
 from typing import (
     Any,
     Dict,
@@ -25,6 +28,7 @@ from typing import (
 
 import attr
 import inflection
+import numpy as np
 from cached_property import cached_property
 from graphql import (
     GraphQLEnumType,
@@ -36,6 +40,7 @@ from graphql import (
     GraphQLType,
     GraphQLUnionType,
 )
+from typing_utils import get_origin
 
 from qctrl.builders.doc import (
     ClassAttribute,
@@ -305,6 +310,7 @@ class TypeRegistry(BaseTypeRegistry):
             # if building non-input type, all sub-fields are
             # optional as it's possible to request partial data
             if not isinstance(obj, GraphQLInputObjectType):
+                is_optional = True
                 field_def.update(dict(default=None))
 
             # otherwise, check if sub field has default value
@@ -322,6 +328,10 @@ class TypeRegistry(BaseTypeRegistry):
                     is_optional = True
                     field_def.update(dict(default=None))
 
+            # add type validator
+            validator = self._get_validator(type_hint, is_optional)
+            field_def.update(dict(validator=validator))
+
             # add field definition
             fields[attr_name] = attr.ib(**field_def)
 
@@ -339,6 +349,45 @@ class TypeRegistry(BaseTypeRegistry):
         cls = self._create_attr_class(obj.name, fields, base, str(docstring))
         return cls
 
+    @staticmethod
+    def _get_validator(type_hint, is_optional):
+        # get origin type from type hints if exists (i.e.: list for List[])
+        base_type = get_origin(type_hint) or type_hint
+        type_checking = [base_type]
+
+        # accepts dict as custom types input
+        if attr.has(base_type):
+            type_checking.append(dict)
+
+        # handle optional attributes
+        if is_optional:
+            type_checking.append(type(None))
+
+        # accept ndarrays where list type is accepted
+        if list in type_checking:
+            type_checking.append(np.ndarray)
+            type_checking.append(tuple)
+
+        # accept float values where complex type are accepted
+        if complex in type_checking:
+            type_checking.append(float)
+            type_checking.append(np.complexfloating)
+
+        # accept int values where float type are accepted
+        if float in type_checking:
+            type_checking.append(int)
+            type_checking.append(np.floating)
+
+        if int in type_checking:
+            type_checking.append(np.integer)
+
+        # Enum values can be set as string
+        if isinstance(type_hint, EnumMeta):
+            type_checking.append(str)
+
+        validator = attr.validators.instance_of(tuple(type_checking))
+        return validator
+
     @staticmethod
     def _get_submodule(namespace: List[str]) -> str:
         submodule = "types"
