@@ -218,12 +218,7 @@ def activity(
                 click.secho(value) for value in activity_monitor.get_valid_types()
             ]
     else:
-
-        click.secho(
-            activity_monitor(
-                limit=limit, offset=offset, status=status, action_type=type
-            )
-        )
+        activity_monitor(limit=limit, offset=offset, status=status, action_type=type)
 
 
 @main.command(help="Displays the current version of the 'qctrl' package.")
